
/////////////////////////////////////////////////////////////////////
// Mapper 234
void NES_mapper234::Reset()
{
  regs[0] = regs[1] = regs[2] = 0;

  set_CPU_banks(0,1,2,3);
  set_PPU_banks(0,1,2,3,4,5,6,7);
}

void NES_mapper234::MemoryWrite(uint32 addr, uint8 data)
{
  switch(addr & 0xFFF8)
  {
    case 0xFF80:
    case 0xFF88:
    case 0xFF90:
    case 0xFF98:
      {
        if(!regs[0])
        {
          regs[0] = data;
          if(data & 0x80)
          {
            set_mirroring(NES_PPU::MIRROR_HORIZ);
          }
          else
          {
            set_mirroring(NES_PPU::MIRROR_VERT);
          }
          if(data & 0x40)
          {
            set_CPU_bank4((data & 0x0E)*4+0);
            set_CPU_bank5((data & 0x0E)*4+1);
            set_CPU_bank6((data & 0x0E)*4+2);
            set_CPU_bank7((data & 0x0E)*4+3);
            set_PPU_bank0(((data & 0x0E)<<2)*8+0);
            set_PPU_bank1(((data & 0x0E)<<2)*8+1);
            set_PPU_bank2(((data & 0x0E)<<2)*8+2);
            set_PPU_bank3(((data & 0x0E)<<2)*8+3);
            set_PPU_bank4(((data & 0x0E)<<2)*8+4);
            set_PPU_bank5(((data & 0x0E)<<2)*8+5);
            set_PPU_bank6(((data & 0x0E)<<2)*8+6);
            set_PPU_bank7(((data & 0x0E)<<2)*8+7);
          }
          else
          {
            set_CPU_bank4((data & 0x0F)*4+0);
            set_CPU_bank5((data & 0x0F)*4+1);
            set_CPU_bank6((data & 0x0F)*4+2);
            set_CPU_bank7((data & 0x0F)*4+3);
            set_PPU_bank0(((data & 0x0F)<<2)*8+0);
            set_PPU_bank1(((data & 0x0F)<<2)*8+1);
            set_PPU_bank2(((data & 0x0F)<<2)*8+2);
            set_PPU_bank3(((data & 0x0F)<<2)*8+3);
            set_PPU_bank4(((data & 0x0F)<<2)*8+4);
            set_PPU_bank5(((data & 0x0F)<<2)*8+5);
            set_PPU_bank6(((data & 0x0F)<<2)*8+6);
            set_PPU_bank7(((data & 0x0F)<<2)*8+7);
          }
        }
      }
      break;

    case 0xFFC0:
    case 0xFFC8:
    case 0xFFD0:
    case 0xFFD8:
      {
        if(!regs[2])
        {
          regs[2] = data;
        }
      }
      break;

    case 0xFFE8:
    case 0xFFF0:
      {
        regs[1] = data;
        if (regs[0] & 0x40)
        {
          set_CPU_bank4(((regs[0] & 0x0E)|(regs[1] & 0x01))*4+0);
          set_CPU_bank5(((regs[0] & 0x0E)|(regs[1] & 0x01))*4+1);
          set_CPU_bank6(((regs[0] & 0x0E)|(regs[1] & 0x01))*4+2);
          set_CPU_bank7(((regs[0] & 0x0E)|(regs[1] & 0x01))*4+3);
          set_PPU_bank0((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+0);
          set_PPU_bank1((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+1);
          set_PPU_bank2((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+2);
          set_PPU_bank3((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+3);
          set_PPU_bank4((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+4);
          set_PPU_bank5((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+5);
          set_PPU_bank6((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+6);
          set_PPU_bank7((((regs[0] & 0x0E)<<2)|((regs[1] & 0x70)>>4))*8+7);
        }
        else
        {
          set_CPU_bank4((regs[0] & 0x0F)*4+0);
          set_CPU_bank5((regs[0] & 0x0F)*4+1);
          set_CPU_bank6((regs[0] & 0x0F)*4+2);
          set_CPU_bank7((regs[0] & 0x0F)*4+3);
          set_PPU_bank0((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+0);
          set_PPU_bank1((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+1);
          set_PPU_bank2((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+2);
          set_PPU_bank3((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+3);
          set_PPU_bank4((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+4);
          set_PPU_bank5((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+5);
          set_PPU_bank6((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+6);
          set_PPU_bank7((((regs[0] & 0x0F)<<2)|((regs[1] & 0x30)>>4))*8+7);
        }
      }
      break;
  }
}
/////////////////////////////////////////////////////////////////////

